#h-annotation-popover-container
  position absolute
  pointer-events none
  margin-left 15px
  z-index 1000

  .h-annotation-popover
    position absolute
    width max-content
    max-width 400px
    background-color #fff
    z-index 1100
    padding 5px 5px 10px
    border-radius 3px
    border 1px solid #000
    box-shadow 0 5px 10px rgba(0, 0, 0, 0.5)

    &:before
      content ""
      border-color transparent #000 transparent transparent
      border-style solid
      border-width 13px
      height 0
      width 0
      position absolute
      left -26px
      top calc(50% - 13px)

    &:after
      content ""
      border-color transparent #fff transparent transparent
      border-style solid
      border-width 13px
      height 0
      width 0
      position absolute
      left -25px
      top calc(50% - 13px)

    > :not(.h-annotation-name)
      padding 0 20px

    > :not(.h-annotation-description)
      white-space nowrap

    .h-annotation-description
      padding-top 10px
      width 400px

    .h-annotation-name
      white-space normal
      font-weight bold

    .h-annotation-creator, .h-annotation-created, .h-annotation-updater, .h-annotation-updated
      font-size 0.9em

      span
        padding-right 5px

    .h-annotation-elements
      font-size 90%

      > table
        padding-left 10px

      > table, .annotation-element-metadata-table > table
        display inline-block

        td:first-child
          font-family monospace
          padding-right 10px

        td:nth-child(2)
          text-overflow ellipsis
          overflow hidden
          max-width 300px

      .h-element-type
        font-weight bold
