.h-zoom-widget
  .h-zoom-slider
    padding 0
    box-shadow none

  .h-zoom-value
    border 1px solid #ccc
    border-radius 2px
    min-width 4em
    padding-left 0
    padding-right 0

  .h-zoom-buttons
    margin-top 10px

  .h-download-button-container
    padding-top 8px
    display flex
    justify-content space-between

  .h-download-button
    border 1px solid #ccc
    border-radius 4px
    min-width 4em
    text-align center
    padding 6px 12px

  .h-download-area-button-selected
    background-color #e6e6e6

  .h-zoom-range-controls
    line-height 0
    display table-cell
    float left
    width 12px
    padding-left 4px

    button
      padding 0
      display inline-block
      font-size 8px

#h-download-submit
  margin-right 8px

#h-msgDisable
  position relative
  right 15%

.btn-group-justified .btn-group>.btn
  padding-left 0
  padding-right 0
