.modal-body
  div.h-plot-resizable
    resize vertical
    overflow auto

    select
      height calc(100% - 24px)

  .select2-container
    display block
    font-size 14px
    line-height 1.42857143
    color #222

  .select2-results__option
    padding 0 6px
