.hui-body
  #g-app-header-container
    #h-navbar-brand
      font-size 25px
      font-weight 700

    .g-header-wrapper
      .h-image-menu-wrapper
        display inline
        float right

    .h-help-button
      padding 15px 7px

    z-index 1000

  @media (min-width 768px)
    .navbar-right
      float none
      margin-right 0
      position absolute
      right 0
      white-space nowrap
      z-index 10

    .navbar-nav > li
      float none
      display inline-block

    .navbar > .container-fluid .navbar-brand
      margin-left 0

    .navbar-text
      float none

  .navbar-header
    z-index 20
    float none
    position absolute
    left 0

  .navbar-header a#h-navbar-brand.navbar-brand
    background-color transparent

  .navbar-default .navbar-nav
    &> .open > a
      color inherit
      background-color rgba(217, 217, 217, 0.5)

    &> .open > a:hover
      color inherit

    &> .open > a:focus
      color inherit

    &> li > a
      color inherit
