.h-frame-selector-widget
  padding 0

  .input-group
    display inherit

  .image-frame-control-box
    & > div
      display flex
      flex-direction column
      padding 0

      &.invisible
        display none

    .icon-keyboard
      display none

  .image-frame-simple-control
    padding 0

  label[for="mode"]
    display none

  table
    margin-bottom 0

  .table-container
    max-height unset
    overflow-y unset

  .table-container td, .table-container th
    padding 0 5px 0 0

  .table-row
    height unset
    vertical-align unset
    border-bottom 1px solid #ddd !important

  td
    padding 2px

  .current-color, .color-col
    width 25px

  .color-col
    padding unset

  .enable-col
    width initial

  td.name-col
    display flex
    flex-direction column
    width 140px
    max-width unset
    overflow hidden
    padding 0

  tr.dual-controls
    td label[for="numberControl"]
      max-width 100px
      overflow hidden

    td input[name="numberControl"]
      margin-right 5px

  .picker-offset
    margin-right 50px
    right 0

  .enabled-col
    font-size 0
    text-align center

  input[type=checkbox]
    width 20px

  #composite-layer-table input[type="number"]
    padding 0
    width 60px

  .range-editor
    input[type=number]
      padding 0
      width 60px

    .handles-svg
      left 60px
      width calc(100% - 120px)

    .canvas
      width calc(100% - 120px)
      left 65px

    .percentage-input
      width unset

    .percentage-input input
      width 60px

    .percentage-input:after
      left 42px

  .auto-range-col
    min-width unset
    display inline-block

  .switch
    width 45px

  .onoff-slider.checked:before
    transform translateX(22px)

  .expand-btn
    font-size unset
    font-weight unset

  .auto-range-label > .small-text
    display none

  .auto-range-label
    min-width 60px
    display block

    .percentage-input input
      width 60px

    .percentage-input:after
      left 42px

  .expand-btn
    right 0

  #composite-layer-table .auto-range-col .percentage-input input
    width 60px

  .presets-menu
    font-size 0

    *
      font-size 14px

  .max-merge-cell
    min-width initial

    label
      display none
