ul.h-analyses-dropdown
  min-width 120px

  li > a
    padding 3px 20px

  .dropdown-menu
    min-width 120px

ul.dropdown-menu[row_offset="1"]
  top -26px

ul.dropdown-menu[row_offset="2"]
  top -52px

ul.dropdown-menu[row_offset="3"]
  top -78px

ul.dropdown-menu[row_offset="4"]
  top -104px

ul.dropdown-menu[row_offset="5"]
  top -130px

ul.dropdown-menu[row_offset="6"]
  top -156px

ul.dropdown-menu[row_offset="7"]
  top -182px

ul.dropdown-menu[row_offset="8"]
  top -208px

ul.dropdown-menu[row_offset="9"]
  top -234px

ul.dropdown-menu[row_offset="10"]
  top -260px

ul.dropdown-menu[row_offset="11"]
  top -286px

ul.dropdown-menu[row_offset="12"]
  top -312px

ul.dropdown-menu[row_offset="13"]
  top -338px

ul.dropdown-menu[row_offset="14"]
  top -364px

ul.dropdown-menu[row_offset="15"]
  top -390px

ul.dropdown-menu[row_offset="16"]
  top -416px

ul.dropdown-menu[row_offset="17"]
  top -442px

ul.dropdown-menu[row_offset="18"]
  top -468px

ul.dropdown-menu[row_offset="19"]
  top -494px

ul.dropdown-menu[row_offset="20"]
  top -520px

ul.dropdown-menu[row_offset="21"]
  top -546px

ul.dropdown-menu[row_offset="22"]
  top -572px

ul.dropdown-menu[row_offset="23"]
  top -598px

ul.dropdown-menu[row_offset="24"]
  top -624px

ul.dropdown-menu[row_offset="25"]
  top -650px

ul.dropdown-menu[row_offset="26"]
  top -676px

ul.dropdown-menu[row_offset="27"]
  top -702px

ul.dropdown-menu[row_offset="28"]
  top -728px

ul.dropdown-menu[row_offset="29"]
  top -754px

ul.dropdown-menu[row_offset="30"]
  top -780px
