.h-hide-show-buttons
  border-bottom 1px solid #ececec
  padding-bottom 8px
  margin-bottom 5px
  display flex

  button.btn
    padding 2px 5px
    margin 0
    background none

  .h-annotation-opacity-container, .h-annotation-fill-opacity-container
    flex 1 1 0%
    position relative
    margin 2px 5px

.h-annotation
  padding 2px
  line-height 1.1
  font-weight initial
  margin-left 5px
  border-radius 3px

  span
    cursor pointer

  .h-float-left
    float left
    margin-right 0.5em

  .h-annotation-right
    float right

  a
    color #333

  .h-annotation-name
    width 170px
    display inline-block
    text-overflow ellipsis
    white-space nowrap
    overflow hidden

  span.partial
    -webkit-mask-image linear-gradient(225deg, #000, rgba(0, 0, 0, 0))
    mask-image linear-gradient(225deg, #000, rgba(0, 0, 0, 0))

.h-annotation-group-name
  font-weight bold
  text-align left
  padding 2px
  border-style none
  border-width 0
  white-space nowrap
  overflow hidden

  i
    margin-right 2px
    float left
    color #6b6b6b

  &:hover
    background-color #eee
    cursor pointer

.h-annotation.h-active-annotation
  background-color #c5cae9 !important

.h-annotation.h-highlight-annotation
  background-color #ccc

.h-annotation-toggle
  margin 0
  border-top 1px solid #ececec
  padding-top 8px
  padding-bottom 2px

  label
    margin 0 5px
    padding-top 5px

  .h-create-annotation
    float right

.h-annotation:hover
  background-color #eee

.h-annotation-selector
  .s-panel-title
    .save-mark
      display none

.h-annotation-selector.saving
  .s-panel-title
    .icon-tags
      display none

    .save-mark
      display inherit
