.h-metadata-widget
  .s-panel-title-container
    .s-no-panel-toggle
      position relative

  .g-widget-metadata-header-key-value
    margin-left 5px
    font-size 10px
    text-overflow ellipsis
    overflow hidden
    max-width 100px
    display inline-block
    white-space nowrap
    vertical-align middle

  .g-widget-metadata-row
    min-height 30px
    margin 0

  .g-widget-metadata-key, .g-widget-metadata-value
    vertical-align top
    margin 0

  .g-json-editor
    width 100%
    margin 0
    overflow hidden

    .jsoneditor-menu
      height initial
      min-height 35px

    table.jsoneditor-search
      position initial

  .g-widget-metadata-value
    width 59%

  button.g-widget-metadata-add-button
    float none
    margin -2px 5px -2px 0
    font-size 10px
    color #333
    background-color #fff
    border-color #ccc

  .g-widget-metadata-container .g-widget-metadata-edit-button
    margin 2px 0 0
