.g-hui-quarantine span, .g-hui-quarantine-item span
  font-weight bold
  display inline-block
  width 1em
  margin-left 0.2em
  margin-right 0.2em
  text-align center
  line-height 1em

.g-hui-open-item
  margin-left 10px
