#g-hui-form
  .g-hui-description
    font-size 12px
    margin 0 0 2px

#g-hui-banner-color, #g-hui-brand-color
  display inline
  max-width 160px

#g-hui-help-url, #g-hui-help-tooltip, #g-hui-help-text
  display inline-block
  width calc(100% - 65px)

#g-hui-banner-default-color, #g-hui-brand-default-color, #g-hui-help-default-url, #g-hui-help-default-tooltip, #g-hui-help-default-text
  width 65px
  display inline
  // Vertical-align is useful to avoid an 5px offset in Mozilla
  vertical-align top
