.h-image-body
  .h-image-view-body
    height calc(100vh - 52px)
    width 100%
    position absolute
    top 52px
    left 0

    .h-image-view-container
      height 100%
      width 100%

    .h-image-view-container:focus
      border none

    .h-image-coordinates-container
      position absolute
      right 0
      bottom 0
      background #eee
      padding 0 3px
      border-radius 3px
      z-index 980

  .h-panel-group-left
    overflow-y auto
    overflow-x hidden
    width 300px
    margin-left 0
    padding-bottom 5px

    .s-panel
      margin-left 5px

    .h-control-panel.s-panel-group
      background-color rgba(255, 255, 255, 0.5)
      border 1px solid black
      border-radius 5px

  .h-panel-group-right
    overflow-y auto
    overflow-x hidden
    width 300px
    right 0
    padding-bottom 5px

    .s-panel
      margin-right 5px

  .image-viewer-loading
    position relative
    font-size 48px
    width 48px
    height 48px
    margin-bottom -48px
    left calc(50% - 32px)
    top calc(50% - 32px)

#h-annotation-context-menu
  position absolute
  z-index 1000

#h-pixelmap-context-menu
  position absolute
  z-index 1000

.s-panel-group .s-panel:empty
  border none

.s-panel-group .s-panel-title-container .s-panel-title
  white-space nowrap
  text-overflow ellipsis
  line-height 1em

.s-panel-group .s-panel-title-container .s-panel-controls
  line-height 1em

.geojs-map
  outline none
