.h-elements-container
  width 100%
  max-height 300px
  overflow-y auto
  margin-top 5px
  padding 3px 5px

  .h-element
    border-radius 3px

    span
      cursor pointer

    .h-element-label
      margin-left 5px
      white-space nowrap
      overflow hidden
      display inline-block
      vertical-align top
      max-width calc(100% - 75px)
      text-overflow ellipsis

    .h-view-element
      margin-left 3px
      display none

    .h-delete-element
      float right

  .h-element.h-highlight-element
    background-color #ccc

  .h-element:hover
    background-color #eee

.h-save-annotation
  float right

.h-draw-widget
  .h-draw-tools
    .btn-group-sm>.btn
      padding-left 3px
      padding-right 7px

    .btn-group-sm>.btn.h-brush-dropdown
      padding-left 0
      padding-right 0

  .h-style-group-row
    margin 10px 0

  .h-group-count
    border-width 1px 0
    border-style solid
    border-color #bababa
    padding 5px 4px 0 3px
    margin 5px 5px 0
    width fit-content
    max-width calc(100% - 10px)

    .h-group-count-label
      display inline-block
      vertical-align top

    .h-group-count-options
      display inline-block
      padding-left 5px
      max-width 200px

      .h-group-count-option
        white-space nowrap
        overflow hidden
        text-overflow ellipsis

    .h-group-count-select
      padding-left 10px

  .btn-default.active
    background-color #5790ff

    &:focus, &:hover
      background-color #4672cc

  .h-panel-name:before
    content " - "

  .h-panel-name
    display inline
    overflow hidden
    white-space nowrap
    text-overflow ellipsis
    font-weight normal

  button[data-type="brush"]
    span.shape
      display none

  button[data-type="brush"][shape="square"]
    span.shape
      &.square
        display inline

  button[data-type="brush"][shape="circle"]
    span.shape
      &.circle
        display inline

  .h-dropdown-title i
    pointer-events none

  .h-dropdown-content
    position absolute
    background white
    border 2px solid black
    border-radius 5px
    right 0
    top 30px
    padding 5px 10px

    .form-group
      margin-bottom 0
      padding 0

      input[type="radio"]
        margin-top 2px

      input[type="number"]
        margin-left 5px

      input[type="checkbox"]
        margin-right 5px

  .h-brush-controls
    width 165px

  .h-brush-size
    width 60px

  .h-fixed-shape-controls
    @extend .h-brush-controls
    z-index 10
    padding 0 25%

  .h-fixed-shape-form-group
    height 100%

  .h-fixed-width
    width 50%

  .h-fixed-height
    width 50%

.flattenicon:before
  transform scale(1, 0.67)
