#g-dialog-container
  .h-access
    float left

.hui-info-list-entry
  user-select text

.hui-annotation-metadata.hui-annotation-metadata-dialog
  .s-panel-title-container
    background-color #f0f0f0
    padding 5px 6px
    color #555
    font-size 16px
    font-weight bold
    margin-top 8px
    position relative

    i.icon-up-open, i.icon-down-open
      display none

    .g-widget-metadata-add-button
      margin-top 0

@media (min-width 768px)
  .modal-dialog.hui-save-annotation-dialog
    width inherit

@media (min-width 900px)
  .modal-dialog.hui-save-annotation-dialog
    width 70%
    max-width 900px

label.unstyled
  margin-bottom unset
  font-weight normal
  margin-left 4px

.h-functional-value
  input[type="number"]
    width 100px
    margin-left 5px

.hui-save-annotation-dialog
  input[type="checkbox"]
    margin-left 5px
