.h-open-annotated-image-dialog
  .media
    .media-left
      min-width 74px

    .media-heading
      white-space nowrap

  .form-inline
    label
      margin 0 5px

  .list-group
    max-height calc(100vh - 250px)
    overflow-y auto
