$headerHeight = 52px

body.hui-body
  user-select none
  overflow hidden

body.hui-body[view-mode="dark"]
  position absolute
  width 100vw
  height 100vh
  background-color #000

  #g-app-header-container, .h-panel-group-right, .h-panel-group-left, #h-annotation-popover-container, #h-annotation-context-menu, #h-pixelmap-context-menu, #g-app-progress-container, #g-dialog-container, #g-alerts-container
    filter invert(100%) hue-rotate(180deg)
    color #111

    img, video, iframe, .geojs-map, .h-colorpicker>.input-group-addon>i
      filter invert(100%) hue-rotate(180deg)

  .h-image-coordinates-container
    background #111
    color #DDD

.h-header-wrapper
  height $headerHeight
  padding-right 20px
  padding-left 20px
  background-color rgba(255, 255, 255, 0.88)
  border-bottom 1px solid #e6e6e6
  transition background-color 0.4s ease-in-out
  position fixed
  top 0
  width 100%
  z-index 10

  &:hover
    background-color white

  .h-user-link
    color #444
    font-size 16px

    &:hover
      color #66a

  .h-login-link-wrapper
    font-size 16px

  .h-user-link-wrapper
    display inline-block

  .h-current-user-wrapper
    float right

  .h-icons-wrapper
    display inline
    line-height $headerHeight
    font-size 21px

    &>a
      color #444
      margin-right 4px
      border 1px solid transparent
      border-radius 3px
      padding 3px

      &:hover
        color #449
        border 1px solid #e2e2e2
        background linear-gradient(top, white 60%, #f6f6f6 100%)

      &:active
        background linear-gradient(bottom, white 60%, #f6f6f6 100%)

#g-app-body-container.h-full-page-body
  margin 0
  margin-top $headerHeight
  min-height 0
  padding 0
  overflow hidden

.h-panel-maximized
  position fixed
  top 50px
  right 0
  bottom 5px
  left 5px
  z-index 10000
  display flex
  flex-direction column
  flex-wrap nowrap

  .s-panel-content
    overflow-y auto
    flex-grow 1

  i.icon-up-open
    display none

#h-analysis-panel
  .s-select-region-button, .s-select-region-submit-button
    &.active, &.enabled
      background-color #5790ff

      &:focus, &:hover
        background-color #4672cc

  .region-dropdown
    position relative
    z-index auto

.modal-dialog .modal-content
  .modal-header, .modal-footer, .modal-body
    padding 8px 15px

  label
    margin-bottom 3px

  .form-group
    margin-bottom 8px
