.h-metadata-plot
  margin 0

  button.g-widget-metadata-plot-settings
    float none
    margin -2px 5px -2px 0
    font-size 13px
    padding 0 1px
    height 21px
    width 22px

  .js-plotly-plot .plotly .modebar-btn
    font-size 13px
    padding 1px 2px

  .h-metadata-plot-area.js-plotly-plot
    min-height 266px
    min-width 266px

  .s-panel-title
    .load-mark
      display none

.h-metadata-plot.loading
  .s-panel-title
    .icon-chart-line
      display none

    .load-mark
      display inherit

.h-panel-maximized
  .h-metadata-plot-area.js-plotly-plot
    height 100%
    width 100%
